package eu.mhsl.craftattack.spawn.appliances.customAdvancements;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliances.customAdvancements.listener.ApplyPendingAdvancementsListener;
import eu.mhsl.craftattack.spawn.appliances.customAdvancements.listener.CustomAdvancementsListener;
import org.bukkit.Bukkit;
import org.bukkit.NamespacedKey;
import org.bukkit.advancement.Advancement;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

public class CustomAdvancements extends Appliance {
    record PendingAdvancement(UUID receiver, String advancement) {
    }

    private final List<PendingAdvancement> pendingAdvancements = new ArrayList<>();

    public void grantAdvancement(String advancementName, UUID playerUUID) {
        Player player = Bukkit.getPlayer(playerUUID);
        if(player == null) {
            addPendingAdvancement(playerUUID, advancementName);
            return;
        }

        try {
            NamespacedKey namespacedKey = Objects.requireNonNull(NamespacedKey.fromString("custom_advancements:craftattack/" + advancementName));
            Advancement advancement = Objects.requireNonNull(Bukkit.getAdvancement(namespacedKey));
            player.getAdvancementProgress(advancement).awardCriteria("criteria");
        } catch (Exception e) {
            Main.logger().info("Custom Advancements Datapack not found!");
        }
    }

    public void applyPendingAdvancements(Player player) {
        if(this.pendingAdvancements.isEmpty()) return;
        List<PendingAdvancement> grantedAdvancements = pendingAdvancements.stream()
            .filter(pendingAdvancement -> pendingAdvancement.receiver.equals(player.getUniqueId())).toList();
        this.pendingAdvancements.removeAll(grantedAdvancements);
        grantedAdvancements.forEach(pendingAdvancement -> grantAdvancement(pendingAdvancement.advancement(), player.getUniqueId()));
    }

    public void addPendingAdvancement(UUID receiver, String advancement) {
        pendingAdvancements.add(new PendingAdvancement(receiver, advancement));
    }

    @Override
    @NotNull
    protected List<Listener> listeners() {
        return List.of(
            new CustomAdvancementsListener(),
            new ApplyPendingAdvancementsListener()
        );
    }
}
